local ADDON_NAME, addon = ...
local oUF = addon.oUF

local function OverrideHealth(self, event, unit)
	if(self.unit ~= unit) then return end
	local health = self.Health

	local min, max = UnitHealth(unit), UnitHealthMax(unit)
	local disconnected = not UnitIsConnected(unit)
	local dead = UnitIsDeadOrGhost(unit)
	health:SetMinMaxValues(0, max)

	if(disconnected) then
		health:SetValue(max)
	elseif(dead) then
		health:SetValue(.001) -- smoothupdate fix
	else
		health:SetValue(min)
	end

	health.disconnected = disconnected

	if(self.ResurrectIcon and (not dead or disconnected)) then
		self.ResurrectIcon:Hide()
	end	

	local r, g, b, t
	if(disconnected) then
		t = self.colors.disconnected
	elseif(UnitIsPlayer(unit)) then
		local _, class = UnitClass(unit)
		t = self.colors.class[class]
	else
		t = self.colors.health
	end

	if(t) then
		r, g, b = t[1], t[2], t[3]	
	end

	if(b) then
		local mu = health.bg.multiplier or 1

		if(self.db.health.reverseClassColors) then
			health:SetStatusBarColor(r * mu, g * mu, b * mu)
			health.bg:SetVertexColor(r, g, b)
		else
			health:SetStatusBarColor(r, g, b)
			health.bg:SetVertexColor(r * mu, g * mu, b * mu)
		end
	end
end

local StyleUpdate = function(self)
	local style = self:GetParent().db.style
	self.db = addon.db.global.styles[style]

	-- Protected Actions
	if(not InCombatLockdown()) then
		self:SetSize(self.db.size.w, self.db.size.h)
	end

	self.Health:SetStatusBarTexture(addon.SM:Fetch("statusbar", self.db.health.statusbar), "BORDER")
	self.Health:ClearAllPoints()
	self.Health:SetPoint("TOP")
	self.Health:SetPoint("LEFT")
	self.Health:SetPoint("RIGHT")

	self.Health.bg:SetTexture(addon.SM:Fetch("statusbar", self.db.health.statusbar))
	self.Health.bg.multiplier = .2

	for k, v in pairs(addon.mods) do
		if(v.update) then
			v.update(self)
		end
	end
end

local Style = function(self, unit)
	self.colors = addon.colors
	self:RegisterForClicks"AnyUp"

	local hp = CreateFrame("StatusBar", nil, self)
	hp.Override = OverrideHealth
	hp.SetValue_ = addon.SetValue 
	hp.SetValue = addon.Smooth
	hp:SetScript("OnUpdate", addon.SmoothUpdate)
	hp.frequentUpdates = true

	self.Health = hp

	local hpbg = hp:CreateTexture(nil, "BACKGROUND")
	hpbg:SetAllPoints(hp)

	self.Health.bg = hpbg

	self.Overlay = CreateFrame("Frame", nil, hp)
	self.Overlay:SetPoint("TOPLEFT", hp)
	self.Overlay:SetPoint("BOTTOMRIGHT", hp)

	for k, v in pairs(addon.mods) do
		if(v.enable) then
			v.enable(self)
		end
	end

	self.PreUpdate = function()
		StyleUpdate(self)
	end
end

local style1 = "Libre"
oUF:RegisterStyle(style1, Style)

addon.combatDelay = CreateFrame"Frame"
addon.combatDelay:SetScript("OnEvent", function(self, ...)
	addon.print("Out of Combat, applying ...")	
	addon:CreateGroups()
	addon:UpdateHeaders()
	addon:UpdateAnchors()
	addon:LibreApply()

	if(addon.resetProfile[addon.profile]) then
		addon.headers[addon.profile] = {}
		addon.anchors[addon.profile] = {}

		addon.resetProfile[addon.profile] = nil
	end

	self:UnregisterEvent("PLAYER_REGEN_ENABLED")
end)

local function UpdateChildren(...)
	for i=1, select("#", ...) do
		local frame = select(i, ...)	
		if(frame) then
			frame:ClearAllPoints()
			frame:UpdateAllElements("RefreshUnit")
		end
	end
end

function addon:LibreApply()
	if(not InCombatLockdown()) then
		addon:UpdateHeaders()

		for k, header in next, addon.headers[addon.profile] do
			UpdateChildren(header:GetChildren())
		end
	else
		addon.combatDelay:RegisterEvent("PLAYER_REGEN_ENABLED")
	end
end

local initConfig = [[
self:SetWidth(%d)
self:SetHeight(%d)
]]

function addon:CreateHeader(db)
	oUF:SetActiveStyle(style1)

	local style = addon.db.global.styles[db.style]
	local anchor = addon:CreateAnchor(db.label)
	local name = addon:randomLabel(ADDON_NAME.."_"..db.label)

	local template, isPet
	if(db.type == "pets") then
		isPet = true
		template = "SecureGroupPetHeaderTemplate"
	else
		template = "SecureGroupHeaderTemplate"
	end

	local spacing = 6

	local header = oUF:SpawnHeader(name, template,
	"hide",
	"showPlayer", true, "showSolo", true, "showParty", true, "showRaid", true,
	"xOffset", spacing,
	"yOffset", -spacing,
	"columnSpacing", spacing,
	"columnAnchorPoint", "LEFT",
	"unitsPerColumn", 5,
	"point", "TOP",
	"maxColumns", 8,
	--"useOwnerUnit", isPet,
	"oUF-initialConfigFunction",
	(initConfig):format(style.size.w, style.size.h))

	header:SetPoint("TOPLEFT", anchor, "TOPLEFT")

	header.db = db
	header.anchor = anchor:GetName()
	table.insert(addon.headers[addon.profile], header)

	if(db.type == "group") then
		local header = {}

		for i=1, 8 do
			header[i] = oUF:SpawnHeader(name..i, template,
			"hide",
			"showParty", true, "showPlayer", true, "showSolo", true, "showRaid", true,
			"xOffset", spacing,
			"yOffset", -spacing,
			"columnSpacing", spacing,
			"groupFilter", i,
			"oUF-initialConfigFunction",
			(initConfig):format(style.size.w, style.size.h, style.size.s))

			if(i == 1) then
				header[i]:SetPoint("TOPLEFT", anchor, "TOPLEFT")
			else
				header[i]:SetPoint("TOPLEFT", header[i-1], "TOPRIGHT", spacing, 0)
			end

			header[i].db = db
			header[i].anchor = anchor:GetName()
			table.insert(addon.headers[addon.profile], header[i])
		end
	end

	addon:UpdateHeaders()
	addon:UpdateAnchors()
end

local conditions = {
	raid = {
		raid = '[group:raid] show;',
		raid10 = '[@raid11,exists] hide;[@raid1,exists] show;',
		raid1025 = '[@raid26,exists] hide;[@raid1,exists] show;',
		raid25 = '[@raid26,exists] hide;[@raid11,exists] show;',
		raid2540 = '[@raid11,exists] show;',
		raid40 = '[@raid26,exists] show;',
	},
	party = '[group:party,nogroup:raid] show;',
	solo = '[@player,exists,nogroup:party] show;',
}

function addon:UpdateHeaders()
	for profile, t in next, addon.headers do
		for k, header in next, t do
			local db = header.db
			local disable

			if(profile == addon.profile) then
				if(db.kill) then
					_G[header.anchor]:Hide()
					addon.db.profile.anchors[header.anchor] = nil
					addon.anchors[addon.profile][header.anchor] = nil
					
					addon.db.profile.groups[db.label] = nil
					t[k] = nil

					disable = true
				else
					local groupNum = tonumber(header:GetAttribute("groupFilter"))

					if(db.unitsPer and not groupNum) then
						header:SetAttribute("unitsPerColumn", db.unitsPer)
					end

					if(db.maxCol and not groupNum) then
						header:SetAttribute("maxColumns", db.maxCol)
					end

					header:SetScale(db.scale)

					if(db.enabled and ((db.keeptogether and groupNum) or 
						(not db.keeptogether and not groupNum))) then

						local visibility = ""
						local visible = db.visible
						for cond in next, visible do
							if(cond == "raid") then
								for x in next, visible.raid do
									visibility = visibility..conditions.raid[x]
								end
							else
								visibility = visibility..conditions[cond]
							end
						end
						visibility = visibility.."hide;"

						RegisterAttributeDriver(header, 'state-visibility', visibility)
					else
						disable = true
					end
				end
			else
				disable = true
			end

			if(disable) then
				RegisterAttributeDriver(header, 'state-visibility', "hide;")
			end
		end
	end
end

